/* ********************************************************************************
(C) Copyright 2004 - Analog Devices, Inc. All rights reserved.

File Name:sportemu_receive.c

Date Modified: 8/24/04

Author: SM 

Software: VisualDSP++3.5
Hardware: BF561 EZ-KIT Board
Purpose: This program will emulate the PCM interface in Multi slot Mode for receving the 
data sent by the Serial Port of BF-561. 
Following are the allocations for the Flags

PF9 - Clk, PF10 - FS, PF11 - DR, PF12 - DT

The data is recevied on the PF11 and stored in the buffer dataBuf.
Hardware Connections are as shown in Figure 7 of the Application note

**********************************************************************************/

#include<defBF561.h>


#define USE_DATA
#define MEASURE_MIPS

.section L1_data_b ;
.var framesyncFlag[4] = {0,0,0x0200,8} ;

.section L1_data_a ;
.var readdataFlag = 0 ;



.byte dataval = 0 ;
.byte oldval = 0 ;

.byte sourceBuf[] =
{

	0x45,
	0x12,
	0x34, 
	0x56, 
	0x78,
	0x12,
	0x34,
	0x22, 
	0x78, 
	0x9A,
	0x12,
	0x34,
	0x56, 
	0x78, 
	0x9A,
	0x12,
	0x34,
	0x56, 
	0x78, 
	0x9A
	
	
} ;

.align 4 ;
.var dataBuf[100];
.var cnt;
.var datawordcnt;

#ifdef LOG_FS
.byte fscnt = 0 ;
#endif

.section L1_code;
.global _main;

_main:


	
	P5.l = dataBuf;
	P5.h = dataBuf;
		
	call __ConfigureFlags;	
	call __SetupInterrupts ;


	
 	call configure_sport;

	//Configure the sport for transmitting the data out
	
	
	
#ifdef MEASURE_MIPS
	r6 = 0 ;
#endif	
	
infinite_loop : jump infinite_loop ;	

_main.END : nop ;


/************************************************************************************************/
// Configure FLAGS	
/************************************************************************************************/
  
.global __ConfigureFlags ;
__ConfigureFlags :

	// Set the direction of flags
    p0.l = lo(FIO0_DIR);		// set PF dir reg
    p0.h = hi(FIO0_DIR);
    r0 = 0x1000 ;				// PF9 - Clk, PF10 - FS, PF11 - DR, PF12 - DT
	w[p0] = r0;
	ssync;
		
	// For clock it is at falling edge. 
	p0.l = lo (FIO0_POLAR) ;
	p0.h = hi (FIO0_POLAR) ;
	r0 = 0x0200 ;
	w[p0] = r0 ;
	ssync;
	
	// Setup interrupt to edge sensitive
	p0.l = lo (FIO0_EDGE) ;
	p0.h = hi (FIO0_EDGE) ;
	r0 = 0x0200 ;
	w[p0] = r0 ;
	ssync;

	// Input enable register to be set
	p0.l = lo (FIO0_INEN) ;
	p0.h = hi (FIO0_INEN) ;
	r0 = 0x0E00 ;
	w[p0] = r0 ;
	ssync;
	
	// Just clear the flag registers
	p0.l = lo (FIO0_FLAG_D) ;
	p0.h = hi (FIO0_FLAG_D) ;
	r0 = 0x0 ;
	w[p0] = r0 ;
	ssync;
	
		
	// Enable the interrupts at the flag level
	// Mapping A to PF9 - Clock
	p0.l = lo (FIO0_MASKA_D) ;
	p0.h = hi (FIO0_MASKA_D) ;
	r0 = 0x0200 ;
	w[p0] = r0 ;		
    ssync;
    
__ConfigureFlags.END :    
    rts ;
    

/************************************************************************************************/
// Setup Interrupts
/************************************************************************************************/
  
.global __SetupInterrupts ;
__SetupInterrupts :
	
	// PF9 - Mapped to IVG11
	
	p0.l = 0x202C;
	p0.h = 0xFFE0;	
	r0.l = ISR_pf9;
	r0.h = ISR_pf9; 		// IVG11 Handler
	[p0++] = r0;
	ssync;
	
    p0.l = lo(IMASK);
    p0.h = hi(IMASK);
    r1 = 0 ;
    r1.l = 0x0800 ;
    [p0] = r1;
    ssync;	
    
    p0.l = lo(SICA_IMASK1);
    p0.h = hi(SICA_IMASK1);
    r0.h = 0x0000 ;
    r0.l = 0x8000 ;
    [p0] = r0 ;
    ssync; 
        
__SetupInterrupts.END : 
	rts ;
	
/************************************************************************************************/
// ISR_pf9
/************************************************************************************************/
.global ISR_pf9 ;
ISR_pf9 :

	
	P3.l = framesyncFlag ;		//initilizing at the begening to avoid initilizing again in the code
	P3.h = framesyncFlag ;
 
// Now check whether we get the frame sync or not Read PF10 first - Store into r7. Dont modify r7

	I1.l = lo (FIO0_FLAG_D) ;	//initilizing at the begening to avoid initilizing again in the code
	I1.h = hi (FIO0_FLAG_D) ;
	
	P4.l = cnt;
 	P4.h = cnt;
	
	r0 = [P3]; 
	
	r7.l = w[I1];		//FIO0_FLAG_D and also checking for FS
	 
	r1.l = 0x0A01; 
	
	r3 = [P3 + 12] || r1 = extract(r7,r1.l)(Z);
	
	[P3] =  r1 ;	
	
	r2 = 1;
	
	CC = r2 == r1;
	
	r1 = 3;
	 
	if !CC jump end;
	
	[P4] = r1;
	
end:	
	
	CC = r0 == 0 ;
		
	if CC jump quit_framesync (BP) ;	
	
	r0 = [p3+4] || [i3] = r3 ;		//Read data flag || Reading the vlaue to initilize frame sync value to zero
	
// Reset the framesync flag to 0 so that we dont care until framesync went
// up again
	p1.l = dataval ;
	p1.h = dataval ;

	
	[P3] = r0 ;			//Frame Sync Flag
						
quit_framesync :	
#ifdef USE_DATA
// You came here either to read data or just to ignore and go further
	
	i3.l = readdataFlag ;
	i3.h = readdataFlag ;
	
    r3 = [P3 +8 ] || r4 = r7 >> 11 || r0 = [i3] ;	// Dont touch r0 any more from here. Same with p0 as well || Reading the Value 0x0200 to write to FIO_FLAG_C
    
   
	CC = r0 == 0 ;
	
// If readdataFlag == 0, dont read any thing
	if CC jump end_dataread;	

			
// Test if a bit is set or not


	r0 = r0 - r2(S) ||r1 = b[p1] (Z); //dataval
	 [i3] = r0 || r1 = r1 << 1 ;
	r1 = r1 | r4 ;
	
// Now r1 has the value left shifted Write it back to dataval
	
	b[p1] = r1 ;	
	
	CC = r0 == 0;
		
	if !CC jump end_dataread ;	
// Final data word written to memory
	r5 = [P4];
	CC = r5 == r0;
	
	if CC jump  end_datareadf;
	r5 += -1;
	[P4] = r5;
	r3 = [P3 + 12];
	r3 += 1;
	[i3] =  r3;
	
	
	B[P5++] = r1 ;
	
//Logic for counting the total number of words;
	r1 = 20;
	
	P2.l = datawordcnt;
	P2.h = datawordcnt;
	
	r0 = [P2]; 
	
	r0 += 1;
	CC = r0 == r1;
	
	if CC jump enddataread;
	
	
	[P2] = r0; 
	
	
end_datareadf:
	
// Once final data word is written to memory, set the 
#endif//USE_DATA		
			
end_dataread :	
// Clear the interrupt first here
	I1.l = lo (FIO0_FLAG_C) ;
	w[I1] = r3.l ;
	ssync;

ISR_pf9.END :
	rti ;
	

enddataread:
	p0.l = lo(IMASK);
    p0.h = hi(IMASK);
    r1 = 0 ;
    [p0] = r1;
    ssync;	
jump enddataread;
		
		
/* ********************************************************************************
(C) Copyright 2004 - Analog Devices, Inc. All rights reserved.

File Name:Transmitsw.c

Date Modified: 8/24/04

Author: SM 

Software: VisualDSP++3.5
Hardware: BF561 EZ-KIT Board
Purpose: This program will generate a simple serial port clock,frame sync and data for 
external device to receive
The data from the array SourceBuf is transmitted on the SPORT1 Primary Data Pin.
Hardware Connections are as shown in Figure 7 of the Application note

**********************************************************************************/


.section program;

configure_sport:


	P0.l = 0x2024;
	P0.h = 0xFFE0;
	r0.l = SPORT1TxInterrupt;
	r0.h = SPORT1TxInterrupt;
	[P0] = r0;
 	
	
	p0.l = lo(IMASK);
    p0.h = hi(IMASK);
    r1 = [p0];
    r0 = 0x0200 ;
    r0=r0|r1;
    [p0] = r0;
    ssync;	
 
 
    p0.l = lo(SICA_IMASK0);
    p0.h = hi(SICA_IMASK0);
    r0.h = 0x0400 ;
    r0.l = 0x0000 ;
    [p0] = r0 ;
    ssync; 

   	//setup interrupts for SPORT transmit.END

   	/*Transmit side DMA parameter initialization start*/
   	R0.l = 0x1000;
	P0.h = DMA2_3_CONFIG >> 16;	
	P0.l = DMA2_3_CONFIG & 0xffff;
	W[P0] =R0.l;
	ssync;

	R0.l=sourceBuf;
	R0.h=sourceBuf;
	
	P0.h = DMA2_3_START_ADDR  >> 16;
	P0.l = DMA2_3_START_ADDR  & 0xffff;
	[P0] =R0;
	ssync;	
	
	R0.H = 0x0;
	R0.L = 0x1;
	
	P0.h = DMA2_3_X_MODIFY >> 16;
	P0.l = DMA2_3_X_MODIFY & 0xffff;
	W[P0] =R0;
	ssync;	
	
	R0.l = 20;
	P0.h = DMA2_3_X_COUNT >> 16;	
	P0.l = DMA2_3_X_COUNT & 0xffff;
	W[P0] =R0.l;
	ssync;	

	R0.l = 0x0080;
	P0.h = DMA2_3_CONFIG >> 16;	
	P0.l = DMA2_3_CONFIG & 0xffff;
	W[P0] =R0.l;
	ssync;	
	
/*Transmit side parameter initialization end*/

/*Enable MultiChannel Mode Start*/
	
	
	R0.L = 0x0007;						
	P0.L = SPORT1_MTCS0 & 0xFFFF;
	[P0] = R0;
	ssync;
	
	
	R0.L = 0x0000;						
	P0.L = SPORT1_MCMC1 & 0xFFFF;
	W[P0] = R0;
	ssync;
	R0.L = 0x101C;						
	P0.L = SPORT1_MCMC2 & 0xFFFF;
	W[P0] = R0;
	ssync;
	

/*Enable MultiChannel Mode End*/


			
	p0.l = lo(SPORT1_RCLKDIV);
    p0.h = hi(SPORT1_RCLKDIV);
    r0.h = 0x0;
    r0.l = 59;	// For 1MHz Serial Clock rate
    W[p0] = r0 ;
    ssync; 
    
    p0.l = lo(SPORT1_RFSDIV);
    p0.h = hi(SPORT1_RFSDIV);
    r0.h = 0x0;
    r0.l = 159;	// For 1MHz Serial Clock rate
    W[p0] = r0 ;
    ssync; 
	 
	// Configure serial port control registers
	
	p0.l = lo(SPORT1_TCR2);
    p0.h = hi(SPORT1_TCR2);
    r0.l = 0x0007;	
    W[p0] = r0.l;
    ssync; 
    
    p0.l = lo(SPORT1_TCR1);
    p0.h = hi(SPORT1_TCR1);
    r0.l = 0x0602;	
    W[p0] = r0.l;
    ssync; 
    
    p0.l = lo(SPORT1_RCR2);
    p0.h = hi(SPORT1_RCR2);
    r0.l = 0x0007;	
    W[p0] = r0.l;
    ssync; 
    
    p0.l = lo(SPORT1_RCR1);
    p0.h = hi(SPORT1_RCR1);
    r0.l = 0x0602;	
    W[p0] = r0.l;
    ssync; 
    
    R0.l = 0x0001;
	P0.h = DMA2_3_CONFIG >> 16;	
	P0.l = DMA2_3_CONFIG & 0xffff;
	R1.l = W[P0];
	R0 = R0 | R1;
	W[P0] =R0.l;
	ssync;
	
	p0.l = lo(SPORT1_RCR1);
    p0.h = hi(SPORT1_RCR1);
    r0.l = 0x0001;	
    r1.l = W[p0];
    r0 = r0 | r1;
    W[p0] = r0.l;
    ssync; 	

		
	p0.l = lo(SPORT1_TCR1);
    p0.h = hi(SPORT1_TCR1);
    r0.l = 0x0001;	
    r1.l = W[p0];
    r0 = r0 | r1;
    W[p0] = r0.l;
    ssync; 	


	rts;

configure_sport.end:




SPORT1TxInterrupt:
	
	R0.l = 0x0001;
	P0.h = DMA2_3_IRQ_STATUS >> 16;	
	P0.l = DMA2_3_IRQ_STATUS & 0xffff;
	W[P0] =R0.l;
	ssync;

#if 0	
	r1 = 0x0008;
	
	
	back_1:
	
	p0.l = lo(DMA2_3_IRQ_STATUS);
    p0.h = hi(DMA2_3_IRQ_STATUS);
    r0.l = W[p0];
    ssync; 	

    CC = r0 == r1;
    
    if CC jump back_1;

		
	back:
		
	p0.l = lo(SPORT1_STAT);
    p0.h = hi(SPORT1_STAT);
    r0.l = W[p0];
    ssync; 	

    CC = r0 == r1;
    
    if !CC jump back;

    
	
	p0.l = lo(SPORT1_TCR1);
    p0.h = hi(SPORT1_TCR1);
    r0.l = 0x0000;	
    W[p0] = r0.l;
    ssync;
#endif    


    rti;
SPORT1TxInterrupt.end:    
	

