/* ********************************************************************************
(C) Copyright 2004 - Analog Devices, Inc. All rights reserved.

File Name:Transmitsw.c

Date Modified: 12/9/04

Author: SM 

Software: VisualDSP++3.5
Hardware: BF561 EZ-KIT Board
Purpose: This program will emulate the PCM interface in Short Frame Sync Mode for 
transmitting the data sent to the Serial Port of BF-561. 
Following are the allocations for the Flags

PF9 for emulating Clk, PF10 for emulating FS, PF12 for emulating DT
The data from the array dataVal is transmitted on the GPIO Pin PF12 and received by destBuf.

Hardware Connections are as shown in Figure 1 of the Application note

**********************************************************************************/

#include<defBF561.h>

#define USE_CLOCK
#define USE_SYNC
#define USE_DATA

// Clock is on FLAG9 - PF9
#define FLAG9_OUTPUT 0x0200

// Frame sync is on FLAG10 - PF10
#define FLAG10_OUTPUT 0x0400

// Dataout is on FLAG12 - PF12
#define FLAG12_OUTPUT 0x1000



.section L1_data_a ;
.var clkcnt = 16 ;
.var fscnt = 0 ;
.var datacnt = 0 ;
.var dataValcnt=0;
.byte dataVal[5] = {0x22,0x11,0xEE,0xFF,0x11};
.byte destBuf[5];


.section program;
.global _main;

_main:
	call configure_sport;
	call __ConfigureFlags;
	
	p0.l = lo(FIO0_FLAG_C);		// set PF dir reg
    p0.h = hi(FIO0_FLAG_C);
    r0 = 0;
    w[p0] = r0 ;

	
	p0.l = lo(FIO0_FLAG_T);		// set PF dir reg
    p0.h = hi(FIO0_FLAG_T);
    r0 = FLAG9_OUTPUT ;
    w[p0] = r0 ;

	call __ConfigureTimerInterrupt ;
	call __EnableTimerInterrupt ;
	call __ConfigureTimer1 ;
	

	infinite_loop : jump infinite_loop ;	

_main.END : nop ;

/************************************************************************************************/
// Configure Interrupts
/************************************************************************************************/

.global __ConfigureTimerInterrupt ;
__ConfigureTimerInterrupt :

	p0.l = 0x2018;
	p0.h = 0xFFE0;			// IVG10 Handler
	
	r0.l = isr_timer1;
	r0.h = isr_timer1; 		// IVG10 Handler
	[p0++] = r0;
__ConfigureTimerInterrupt.END :	
	rts ;

/************************************************************************************************/
// Enable Interrupts
/************************************************************************************************/
	
.global __EnableTimerInterrupt ;
__EnableTimerInterrupt :
	
	// enable IVG7  interrupt
    p0.l = lo(IMASK);
    p0.h = hi(IMASK);
    r0 = 0x0;			
    r0.l = 0x0040;	//IVG10
    [p0] = r0;
    ssync;
	    
__EnableTimerInterrupt.END :    
    rts ;

/************************************************************************************************/
// Configure FLAGS
/************************************************************************************************/
  
.global __ConfigureFlags ;
__ConfigureFlags :

    p0.l = lo(FIO0_DIR);		// set PF dir reg
    p0.h = hi(FIO0_DIR);
    
    r0.l =     
#ifdef USE_CLOCK    
	FLAG9_OUTPUT 
#endif
#ifdef USE_SYNC    
	| FLAG10_OUTPUT 
#endif	
#ifdef USE_DATA    
	| FLAG12_OUTPUT 
#endif	
	;

	w[p0] = r0;

    ssync;
    
__ConfigureFlags.END :    
    rts ;
    
   
/************************************************************************************************/
// Configure Timer1
/************************************************************************************************/

.global __ConfigureTimer1 ;
__ConfigureTimer1 :

    p0.l = lo(TCOUNT);
    p0.h = hi(TCOUNT);
    r0 = 0 ;
    r0.l = 37500/2;
    [p0] = r0;
    ssync;

    p0.l = lo(TPERIOD);
    p0.h = hi(TPERIOD);
    r0 = 0;
    r0.l = 37500/2 ;
    [p0] = r0;
    ssync;

    p0.l = lo(TCNTL); 
    p0.h = hi(TCNTL);
    r0 = 0;
    r0.l = 0x0007;
    [p0] = r0;
    ssync;

__ConfigureTimer1.END :    
    rts ;
  
/************************************************************************************************/
// ISR for Timer1 Interrupt
/************************************************************************************************/     
isr_timer1:           // IVG 7 Handler     

#ifdef USE_CLOCK

	p0.l = lo(FIO0_FLAG_T);		// set PF dir reg
    p0.h = hi(FIO0_FLAG_T);
    
    r0.l = FLAG9_OUTPUT;				//PF9 as CLOCK
    w[p0] = r0;
    ssync;
    
    p0.l = clkcnt ;
	p0.h = clkcnt ;
	r1 = [p0] ;
	r1 += -1 ;
	[p0] = r1 ;
	CC = AZ ;

	if !CC jump end_clk ;
	// CC = 0 => reload clkcnt with 18
	r1 = 18 ;
	[p0] = r1 ;
	
end_clk : nop ;	
#endif

#ifdef USE_SYNC
	if !CC jump no_framesync;
		
	p0.l = lo(FIO0_FLAG_C);		// We are clearing this so as to clear the last data
    p0.h = hi(FIO0_FLAG_C);
    r1.l = FLAG12_OUTPUT ;
    w[p0] = r1 ;
    ssync;
    
	// Assert frame sync	
	p0.l = lo(FIO0_FLAG_T);		// set PF dir reg
    p0.h = hi(FIO0_FLAG_T);
    r1.l = FLAG10_OUTPUT ;
    w[p0] = r1 ;
    ssync;
    p0.l = fscnt ;
    p0.h = fscnt ;
    r0 = 2 ;
    [p0] = r0 ;
    jump end_data ;        
   
no_framesync :
	// Check if the frame sync needs to be deasserted
    p0.l = fscnt ;
    p0.h = fscnt ;
    r0 = [p0] ;
    r0 += -1 ;
    CC = AZ ;
    [p0] = r0 ;
    if !CC jump end_framesync;
    // Else deassert frame sync now
	p0.l = lo(FIO0_FLAG_T);		// set PF dir reg
    p0.h = hi(FIO0_FLAG_T);
    r1.l = FLAG10_OUTPUT ;
    w[p0] = r1;
end_framesync :    
    ssync;

#endif

#ifdef USE_DATA

	// Drive data only from the next cycle after frame sync is de-asserted
	if !CC jump process_data ;
	// We are here because frame sync is de-asserted in this cycle
	// From the next cycle onwards drive the data
	p0.l = datacnt ;
	p0.h = datacnt ;
	r0 = 7 ;
	[p0] = r0 ;
	p0.l = dataVal ;
	p0.h = dataVal ;
	p1.l = dataValcnt;
	p1.h = dataValcnt;
	P2 = [p1];
	P0 = P0+P2;  
	r1 = B[p0](Z) ;
	r1 >>= r0 ;
	r2 = 0x1 ;
	r1 = r1 & r2 ;
	
	p0.l = lo(FIO0_FLAG_D);		// set PF dir reg
    p0.h = hi(FIO0_FLAG_D);
    r0 = w[p0] (Z);
    r2 = 0x0e00 ;
    r0 = r0 & r2 ;		//Here we are resoring the vlaues for PF11,PF10,PF9
    r1 = r1 << 12 ;
    r0 = r0 | r1 ;
    w[p0] = r0 ;
		
	r7 = 2 ;	// r7 is used for measure the # of cycles before driving next data
	r5 = 5;
	p0.l = dataValcnt;
	p0.h = dataValcnt;
	r0 = [p0];
	cc = r0 == r5;
	if cc jump end_transfer;
	r0 += 1; 
	[p0] = r0;
	
	jump end_data ;
	
process_data :	
	r7 += -1 ;
	CC = AZ ;
	
	if !CC jump end_data ;
	// I am here because I have to drive new data
	p0.l = datacnt ;
	p0.h = datacnt ;
	r0 = [p0] ;
	r0 += -1 ;
	[p0] = r0 ;
	
	p0.l = dataVal ;
	p0.h = dataVal ;
	p1.l = dataValcnt;
	p1.h = dataValcnt;
	P2 = [p1];
	P0 = P0+P2;  
	r1 = B[p0-1](Z) ;
	r1 >>= r0 ;
	r2 = 0x1 ;
	r1 = r1 & r2 ;

	p0.l = lo(FIO0_FLAG_D);		// set PF dir reg
    p0.h = hi(FIO0_FLAG_D);
    r0 = w[p0] (Z);
    r2 = 0x0e00 ;
    r0 = r0 & r2 ;    
    r1 = r1 << 12 ;
    r0 = r0 | r1 ;
    w[p0] = r0 ;
		
	r7 = 2 ;	// r7 is used for measure the # of cycles before driving next data	
	
end_data : ssync ;	

#endif
	rti;
	
isr_timer1.end:

end_transfer:

	p0.h = hi(IMASK);
    r0 = 0x0;			
    [p0] = r0;
    ssync;

end_transfer.end:
	jump end_transfer.end;
	
	
	
	
/* ********************************************************************************
(C) Copyright 2004 - Analog Devices, Inc. All rights reserved.

File Name:Receivesw.c

Date Modified: 12/9/04

Author: SM 

Software: VisualDSP++3.5
Hardware: BF561 EZ-KIT Board
Purpose: This program will will take input clock,frame sync and data which is generated 
by emulating the GPIOs.
The data from the array dataVal is transmitted on the GPIO Pin PF12 and received by destBuf.
Hardware Connections are as shown in Figure 1 of the Application note

**********************************************************************************/


.section program;

configure_sport:

	[--sp] = r1;
	[--sp] = r0;
	[--sp] = p0;


	P0.l = 0x2024;
	P0.h = 0xFFE0;
	r0.l = SPORT1RxInterrupt;
	r0.h = SPORT1RxInterrupt;
	[P0] = r0;
 	
	p0.l = lo(IMASK);
    p0.h = hi(IMASK);
    r1 = [p0];
    r0 = 0x0200 ;
    r0=r0|r1;
    [p0] = r0;
    ssync;	
 
 
    p0.l = lo(SICA_IMASK0);
    p0.h = hi(SICA_IMASK0);
    r0.h = 0x0200 ;
    r0.l = 0x0000 ;
    [p0] = r0 ;
    ssync; 

   	//setup interrupts for SPORT transmit.END

   	/*Transmit side DMA parameter initialization start*/
   	R0.l = 0x1000;
	P0.h = DMA2_2_CONFIG >> 16;	
	P0.l = DMA2_2_CONFIG & 0xffff;
	W[P0] =R0.l;
	ssync;

	R0.l=destBuf;
	R0.h=destBuf;
	
	P0.h = DMA2_2_START_ADDR  >> 16;
	P0.l = DMA2_2_START_ADDR  & 0xffff;
	[P0] =R0;
	ssync;	
	
	R0.H = 0x0;
	R0.L = 0x1;
	
	P0.h = DMA2_2_X_MODIFY >> 16;
	P0.l = DMA2_2_X_MODIFY & 0xffff;
	W[P0] =R0;
	ssync;	
	
	R0.l = 5;
	P0.h = DMA2_2_X_COUNT >> 16;	
	P0.l = DMA2_2_X_COUNT & 0xffff;
	W[P0] =R0.l;
	ssync;	

	R0.l = 0x0082;
	P0.h = DMA2_2_CONFIG >> 16;	
	P0.l = DMA2_2_CONFIG & 0xffff;
	W[P0] =R0.l;
	ssync;	
	
/*Receive side parameter initialization end*/
//External clock and frame sync.

	 
	// Configure serial port control registers
	
	p0.l = lo(SPORT1_RCR2);
    p0.h = hi(SPORT1_RCR2);
    r0.l = 0x0007;	
    W[p0] = r0.l;
    ssync; 
    
    p0.l = lo(SPORT1_RCR1);
    p0.h = hi(SPORT1_RCR1);
    r0.l = 0x0400;	
    W[p0] = r0.l;
    ssync; 
    
    R0.l = 0x0001;
	P0.h = DMA2_2_CONFIG >> 16;	
	P0.l = DMA2_2_CONFIG & 0xffff;
	R1.l = W[P0];
	R0 = R0 | R1;
	W[P0] =R0.l;
	ssync;
	
	p0.l = lo(SPORT1_RCR1);
    p0.h = hi(SPORT1_RCR1);
    r0.l = 0x0001;	
    r1.l = W[p0];
    r0 = r0 | r1;
    W[p0] = r0.l;
    ssync; 	

    p0 =[sp++];
    r0 =[sp++];
    r1 =[sp++];
	rts;

configure_sport.end:




SPORT1RxInterrupt:
	
	R0.l = 0x0001;
	P0.h = DMA2_2_IRQ_STATUS >> 16;	
	P0.l = DMA2_2_IRQ_STATUS & 0xffff;
	W[P0] =R0.l;
	ssync;

    rti;
SPORT1RxInterrupt.end:    
	
